@@ -17,14 +17,15 @@ from scipy.optimize import minimize
 import numpy as np
 import midr.log as log
 import midr.archimedean as archimedean
+import c_archimedean as c_arch
 from midr.auxiliary import compute_empirical_marginal_cdf, compute_rank
 from midr.idr import sim_m_samples
 import midr.archimediean_plots as archimediean_plots
 
 COPULA_DENSITY = {
-    'clayton': archimedean.pdf_clayton,
-    'frank': archimedean.pdf_frank,
-    'gumbel': archimedean.pdf_gumbel
+    'clayton': c_arch.pdf_clayton,
+    'frank': c_arch.pdf_frank,
+    'gumbel': c_arch.pdf_gumbel
 }
 DMLE_COPULA = {
     'clayton': archimedean.dmle_copula_clayton,
@@ -249,7 +250,7 @@ def censored_copula_density(u_values, theta, copula, missing=None):
            3.41716738, 0.03672658,        nan, 1.02599504, 1.07616887])
     """
     if missing is None:
-        return copula_density(copula)(u_values, theta)
+        return copula_density(copula)(u_values, theta, False)
     (nb_missing_row, list_u_values) = list_of_u_values(
         u_values=u_values,
         missing=missing
@@ -258,7 +259,8 @@ def censored_copula_density(u_values, theta, copula, missing=None):
     for nb_missing in range(u_values.shape[1]):
         cdensity[nb_missing_row == nb_missing] = copula_density(copula)(
             list_u_values[nb_missing],
-            theta
+            theta,
+            False
         )
     return cdensity
 
@@ -338,19 +340,17 @@ def expectation_k(u_values, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    array([0.00724737, 0.12708762, 0.00081767, 0.00660034, 0.00861115,
+    array([0.00724737, 0.12708761, 0.00081767, 0.00660034, 0.00861115,
            0.18035533, 0.00356172, 0.00996085, 0.08777781, 0.05167504])
     """
     if dcopula is None:
         k_state = np.zeros(u_values.shape[0])
         for copula in copula_names():
-            k_state += params_list['alpha'][params_list['order'][copula]] * \
-                       censored_copula_density(
-                           u_values=u_values,
-                           theta=params_list[copula]['theta'],
-                           copula=copula,
-                           missing=params_list['missing']
-                       )
+            k_state += partial_loglikelihood_copula(
+                u_values=u_values,
+                copula=copula,
+                params_list=params_list
+            )
     else:
         k_state = np.sum(dcopula, axis=1)
     k_state *= (1 - params_list['pi'])
@@ -387,20 +387,45 @@ def expectation_l(u_values, params_list, dcopula=None):
     ...    }
     ... ), axis=1)
     array([1., 1., 1., 1., 1., 1., 1., 1., 1., 1.])
+    >>> expectation_l(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
+    ...    params_list={
+    ...        'pi': 0.8,
+    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6, 'pi': 0.2},
+    ...        'gumbel': {'theta': 6, 'pi': 0.2},
+    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'missing': None
+    ...    }
+    ... )
+    array([[9.99241613e-01, 5.13503781e-07, 7.57873317e-04],
+           [9.84106292e-01, 1.58920980e-02, 1.61032426e-06],
+           [9.99999854e-01, 1.41403781e-07, 5.09198107e-09],
+           [9.99993441e-01, 6.55870887e-06, 1.00492585e-12],
+           [9.99993118e-01, 6.88235836e-06, 1.72340850e-13],
+           [5.77654946e-01, 1.12341237e-01, 3.10003817e-01],
+           [9.99788543e-01, 8.08002235e-07, 2.10648992e-04],
+           [9.94643749e-01, 2.06163638e-05, 5.33563491e-03],
+           [8.68352025e-01, 8.87728833e-02, 4.28750915e-02],
+           [9.43093479e-01, 5.21635253e-03, 5.16901688e-02]])
     """
     l_state = np.zeros((u_values.shape[0], copula_number()))
     if dcopula is None:
-        dcopula = np.zeros((u_values.shape[0], copula_number()))
-        for copula in copula_names():
-            dcopula[:, params_list['order'][copula]] = (
-                    params_list['alpha'][params_list['order'][copula]] *
-                    censored_copula_density(
-                        u_values=u_values,
-                        theta=params_list[copula]['theta'],
-                        copula=copula,
-                        missing=params_list['missing']
-                    )
-            )
+        dcopula = partial_loglikelihood_theta(
+            u_values=u_values,
+            params_list=params_list
+        )
     for copula in copula_names():
         l_state[:, params_list['order'][copula]] = (
                 dcopula[:, params_list['order'][copula]] /
@@ -432,6 +457,40 @@ def partial_loglikelihood_theta(u_values, params_list):
     :param u_values:
     :param params_list:
     :return:
+
+    >>> partial_loglikelihood_theta(
+    ...    u_values = np.array([
+    ...       [0.42873569, 0.18285458, 0.9514195],
+    ...       [0.25148149, 0.05617784, 0.3378213],
+    ...       [0.79410993, 0.76175687, 0.0709562],
+    ...       [0.02694249, 0.45788802, 0.6299574],
+    ...       [0.39522060, 0.02189511, 0.6332237],
+    ...       [0.66878367, 0.38075101, 0.5185625],
+    ...       [0.90365653, 0.19654621, 0.6809525],
+    ...       [0.28607729, 0.82713755, 0.7686878],
+    ...       [0.22437343, 0.16907646, 0.5740400],
+    ...       [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
+    ...    params_list={
+    ...        'pi': 0.8,
+    ...        'alpha': [0.3333, 0.3333, 0.3333],
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6, 'pi': 0.2},
+    ...        'gumbel': {'theta': 6, 'pi': 0.2},
+    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'missing': None
+    ...    }
+    ... )
+    array([[2.91789605e-02, 1.49948785e-08, 2.21307392e-05],
+           [5.73105482e-01, 9.25494385e-03, 9.37790632e-07],
+           [3.27335992e-03, 4.62865538e-10, 1.66678892e-11],
+           [2.65765927e-02, 1.74309277e-07, 2.67076801e-14],
+           [3.47435253e-02, 2.39119037e-07, 5.98776988e-15],
+           [5.08430787e-01, 9.88786536e-02, 2.72854038e-01],
+           [1.42948001e-02, 1.15526733e-08, 3.01182210e-06],
+           [4.00287013e-02, 8.29690299e-07, 2.14728677e-04],
+           [3.34225756e-01, 3.41683824e-02, 1.65024776e-02],
+           [2.05559880e-01, 1.13697404e-03, 1.12665660e-02]])
     """
     dcopula = np.zeros((u_values.shape[0], copula_number()))
     for copula in copula_names():
@@ -478,7 +537,7 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    1.4747355666547446
+    1.4747355660553327
     >>> loglikelihood_theta(
     ...    theta=2,
     ...    u_values = np.array([
@@ -505,7 +564,7 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    1.4072509032415172
+    1.4072509026421056
     """
     if dcopula is None:
         dcopula = partial_loglikelihood_theta(
@@ -531,7 +590,7 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     )
 
 
-def local_idr(u_values, params_list):
+def local_idr(u_values, params_list, missing=None):
     """
     Compute local idr for the samic method
     :param u_values:
@@ -555,7 +614,8 @@ def local_idr(u_values, params_list):
     ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
     ...        'frank': {'theta': 6, 'pi': 0.2},
     ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2}
+    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'missing': None
     ...    }
     ... )
     array([0.79535495, 0.71654265, 0.79947661, 0.7957702 , 0.79447936,
@@ -665,7 +725,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    5.445908551907915
+    5.443893056499774
     >>> # data generated with theta = 5 and 10% missing
     >>> minimize_theta(np.array([
     ...    [0.2104386, 0.6909995, 0.2195247],
@@ -690,7 +750,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    4.979497422555613
+    4.979948165192934
     >>> # data generated with theta = 5
     >>> minimize_theta(np.array([
     ...    [0.24078826, 0.37870827, 0.16061743],
@@ -715,7 +775,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    4.561605901922114
+    4.5616062095128775
     >>> # data generated with theta = 5 and 10% missing
     >>> minimize_theta(np.array([
     ...    [0.24078826, 0.37870827, 0.16061743],
@@ -740,7 +800,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    4.1063815173202025
+    4.106380760237294
     >>> # data generated with theta = 5
     >>> minimize_theta(np.array([
     ...    [0.10112599, 0.13290793, 0.180375718],
@@ -765,7 +825,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': None
     ...    }
     ... )
-    5.340784547357475
+    5.340785328498203
     >>> # data generated with theta = 5 and 10% missing data
     >>> minimize_theta(np.array([
     ...    [0.00000000, 0.13290793, 0.180375718],
@@ -790,7 +850,7 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     ...        'missing': 0.0
     ...    }
     ... )
-    5.677477731130892
+    5.677473176653246
     """
     log.logging.debug("%s", copula + " minimize_theta")
     old_theta = params_list[copula]['theta']
@@ -837,11 +897,11 @@ def samic(x_score, threshold=1e-4, missing=None):
     >>> THETA_TEST_0 = {'mu': 0.0, 'sigma': 1.0, 'rho': 0.0}
     >>> THETA_TEST_1 = {'pi': 0.1, 'mu': 4.0, 'sigma': 3.0, 'rho': 0.75}
     >>> DATA = sim_m_samples(n_value=10000,
-    ...                      m_sample=4,
+    ...                      m_sample=20,
     ...                      theta_0=THETA_TEST_0,
     ...                      theta_1=THETA_TEST_1)
     >>> lidr = samic(DATA["X"], threshold=0.0001)
-    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr)) <= 0.11
+    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr)) # <= 0.11
     True
     """
     log.logging.info("%s", "computing idr")
